"""Generated message classes for cloudnumberregistry version v1alpha.

"""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding
from apitools.base.py import extra_types


package = 'cloudnumberregistry'


class AggregatedData(_messages.Message):
  r"""Aggregated data for the RegistryBook.

  Fields:
    customRangesCount: Output only. Number of CustomRanges in the
      RegistryBook.
    customRealmsCount: Output only. Number of custom Realms in the
      RegistryBook.
    discoveredRangesCount: Output only. Number of DiscoveredRanges in the
      RegistryBook.
    discoveredRealmsCount: Output only. Number of discovered Realms in the
      RegistryBook.
    uniqueScopesCount: Output only. Number of scopes unique to the
      RegistryBook.
  """

  customRangesCount = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  customRealmsCount = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  discoveredRangesCount = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  discoveredRealmsCount = _messages.IntegerField(4, variant=_messages.Variant.INT32)
  uniqueScopesCount = _messages.IntegerField(5, variant=_messages.Variant.INT32)


class Attribute(_messages.Message):
  r"""A key-value pair representing a custom attribute associated with a
  resource.

  Fields:
    key: Required. The key of the attribute.
    value: Required. The value of the attribute.
  """

  key = _messages.StringField(1)
  value = _messages.StringField(2)


class CancelOperationRequest(_messages.Message):
  r"""The request message for Operations.CancelOperation."""


class CheckAvailabilityIpamAdminScopesResponse(_messages.Message):
  r"""Response message for the
  CloudNumberRegistry.CheckAvailabilityIpamAdminScopes method.

  Fields:
    scopeAvailabilities: The details of the requested scopes.
  """

  scopeAvailabilities = _messages.MessageField('IpamAdminScopeAvailability', 1, repeated=True)


class CleanupIpamAdminScopeRequest(_messages.Message):
  r"""Request message for the CloudNumberRegistry.CleanupIpamAdminScope
  method.

  Fields:
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes after the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  requestId = _messages.StringField(1)


class CloudnumberregistryProjectsLocationsCustomRangesCreateRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsCustomRangesCreateRequest object.

  Fields:
    customRange: A CustomRange resource to be passed as the request body.
    customRangeId: Required. The ID to use for the CustomRange, which will
      become the final segment of the resource name.
    parent: Required. The parent resource name where the CustomRange will be
      created.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  customRange = _messages.MessageField('CustomRange', 1)
  customRangeId = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)
  requestId = _messages.StringField(4)


class CloudnumberregistryProjectsLocationsCustomRangesDeleteRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsCustomRangesDeleteRequest object.

  Fields:
    force: Optional. If set to true, all associated resources will be deleted.
    name: Required. The resource name of the CustomRange to delete.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes after the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  force = _messages.BooleanField(1)
  name = _messages.StringField(2, required=True)
  requestId = _messages.StringField(3)


class CloudnumberregistryProjectsLocationsCustomRangesFindFreeIpRangesRequest(_messages.Message):
  r"""A
  CloudnumberregistryProjectsLocationsCustomRangesFindFreeIpRangesRequest
  object.

  Fields:
    cidrPrefixLength: Required. The prefix length of the free IP ranges to
      find.
    name: Required. The resource name of the CustomRange to search within.
    rangeCount: Optional. The number of free IP ranges to find.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  cidrPrefixLength = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  name = _messages.StringField(2, required=True)
  rangeCount = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  requestId = _messages.StringField(4)


class CloudnumberregistryProjectsLocationsCustomRangesGetRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsCustomRangesGetRequest object.

  Fields:
    name: Required. The resource name of the CustomRange to retrieve.
  """

  name = _messages.StringField(1, required=True)


class CloudnumberregistryProjectsLocationsCustomRangesListRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsCustomRangesListRequest object.

  Fields:
    filter: Optional. Filter expression to filter the results.
    orderBy: Optional. Hint for how to order the results.
    pageSize: Optional. Requested page size. Server may return fewer items
      than requested. If unspecified, server will pick an appropriate default.
    pageToken: Optional. A token identifying a page of results the server
      should return.
    parent: Required. The parent resource name, for example
      `projects/*/locations/*`.
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class CloudnumberregistryProjectsLocationsCustomRangesPatchRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsCustomRangesPatchRequest object.

  Fields:
    customRange: A CustomRange resource to be passed as the request body.
    name: Required. Identifier. The resource name of the CustomRange, in the
      format
      `projects/{project}/locations/{location}/customRanges/{custom_range}`.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
    updateMask: Optional. Field mask is used to specify the fields to be
      overwritten in the CustomRange resource by the update. The fields
      specified in the update_mask are relative to the resource, not the full
      request. A field will be overwritten if it is in the mask. If the user
      does not provide a mask then all fields will be overwritten.
  """

  customRange = _messages.MessageField('CustomRange', 1)
  name = _messages.StringField(2, required=True)
  requestId = _messages.StringField(3)
  updateMask = _messages.StringField(4)


class CloudnumberregistryProjectsLocationsCustomRangesShowUtilizationRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsCustomRangesShowUtilizationRequest
  object.

  Fields:
    name: Required. The resource name of the CustomRange.
  """

  name = _messages.StringField(1, required=True)


class CloudnumberregistryProjectsLocationsDiscoveredRangesFindFreeIpRangesRequest(_messages.Message):
  r"""A
  CloudnumberregistryProjectsLocationsDiscoveredRangesFindFreeIpRangesRequest
  object.

  Fields:
    cidrPrefixLength: Required. The prefix length of the free IP ranges to
      find.
    name: Required. The resource name of the DiscoveredRange to search within.
    rangeCount: Optional. The number of free IP ranges to find.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  cidrPrefixLength = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  name = _messages.StringField(2, required=True)
  rangeCount = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  requestId = _messages.StringField(4)


class CloudnumberregistryProjectsLocationsDiscoveredRangesGetRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsDiscoveredRangesGetRequest object.

  Fields:
    name: Required. The resource name of the DiscoveredRange to retrieve.
  """

  name = _messages.StringField(1, required=True)


class CloudnumberregistryProjectsLocationsDiscoveredRangesListRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsDiscoveredRangesListRequest
  object.

  Fields:
    filter: Optional. Filter expression to filter the results.
    orderBy: Optional. Hint for how to order the results.
    pageSize: Optional. Requested page size. Server may return fewer items
      than requested. If unspecified, server will pick an appropriate default.
    pageToken: Optional. A token identifying a page of results the server
      should return.
    parent: Required. The parent resource name, for example
      `projects/*/locations/*`.
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class CloudnumberregistryProjectsLocationsDiscoveredRangesShowUtilizationRequest(_messages.Message):
  r"""A
  CloudnumberregistryProjectsLocationsDiscoveredRangesShowUtilizationRequest
  object.

  Fields:
    name: Required. The resource name of the DiscoveredRange.
  """

  name = _messages.StringField(1, required=True)


class CloudnumberregistryProjectsLocationsGetRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsGetRequest object.

  Fields:
    name: Resource name for the location.
  """

  name = _messages.StringField(1, required=True)


class CloudnumberregistryProjectsLocationsIpamAdminScopesCheckAvailabilityRequest(_messages.Message):
  r"""A
  CloudnumberregistryProjectsLocationsIpamAdminScopesCheckAvailabilityRequest
  object.

  Fields:
    parent: Required. The parent resource name, for example
      `projects/*/locations/*`.
    scopes: Required. The administrative scopes to check for availability.
  """

  parent = _messages.StringField(1, required=True)
  scopes = _messages.StringField(2, repeated=True)


class CloudnumberregistryProjectsLocationsIpamAdminScopesCleanupRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsIpamAdminScopesCleanupRequest
  object.

  Fields:
    cleanupIpamAdminScopeRequest: A CleanupIpamAdminScopeRequest resource to
      be passed as the request body.
    name: Required. The resource name of the IpamAdminScope to clean up.
  """

  cleanupIpamAdminScopeRequest = _messages.MessageField('CleanupIpamAdminScopeRequest', 1)
  name = _messages.StringField(2, required=True)


class CloudnumberregistryProjectsLocationsIpamAdminScopesCreateRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsIpamAdminScopesCreateRequest
  object.

  Fields:
    ipamAdminScope: A IpamAdminScope resource to be passed as the request
      body.
    ipamAdminScopeId: Required. The ID to use for the IpamAdminScope, which
      will become the final segment of the resource name.
    parent: Required. The parent resource name where the IpamAdminScope will
      be created.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  ipamAdminScope = _messages.MessageField('IpamAdminScope', 1)
  ipamAdminScopeId = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)
  requestId = _messages.StringField(4)


class CloudnumberregistryProjectsLocationsIpamAdminScopesDeleteRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsIpamAdminScopesDeleteRequest
  object.

  Fields:
    force: Optional. If set to true, all associated resources will be deleted.
    name: Required. The resource name of the IpamAdminScope to delete.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes after the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  force = _messages.BooleanField(1)
  name = _messages.StringField(2, required=True)
  requestId = _messages.StringField(3)


class CloudnumberregistryProjectsLocationsIpamAdminScopesDisableRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsIpamAdminScopesDisableRequest
  object.

  Fields:
    disableIpamAdminScopeRequest: A DisableIpamAdminScopeRequest resource to
      be passed as the request body.
    name: Required. The resource name of the IpamAdminScope to disable.
  """

  disableIpamAdminScopeRequest = _messages.MessageField('DisableIpamAdminScopeRequest', 1)
  name = _messages.StringField(2, required=True)


class CloudnumberregistryProjectsLocationsIpamAdminScopesGetRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsIpamAdminScopesGetRequest object.

  Fields:
    name: Required. The resource name of the IpamAdminScope to retrieve.
  """

  name = _messages.StringField(1, required=True)


class CloudnumberregistryProjectsLocationsIpamAdminScopesListRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsIpamAdminScopesListRequest object.

  Fields:
    filter: Optional. Filter expression to filter the results.
    orderBy: Optional. Hint for how to order the results.
    pageSize: Optional. Requested page size. Server may return fewer items
      than requested. If unspecified, server will pick an appropriate default.
    pageToken: Optional. A token identifying a page of results the server
      should return.
    parent: Required. The parent resource name, for example
      `projects/*/locations/*`.
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class CloudnumberregistryProjectsLocationsIpamAdminScopesPatchRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsIpamAdminScopesPatchRequest
  object.

  Fields:
    ipamAdminScope: A IpamAdminScope resource to be passed as the request
      body.
    name: Required. Identifier. The resource name of the IpamAdminScope.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
    updateMask: Optional. Field mask is used to specify the fields to be
      overwritten in the IpamAdminScope resource by the update. The fields
      specified in the update_mask are relative to the resource, not the full
      request. A field will be overwritten if it is in the mask. If the user
      does not provide a mask then all fields will be overwritten.
  """

  ipamAdminScope = _messages.MessageField('IpamAdminScope', 1)
  name = _messages.StringField(2, required=True)
  requestId = _messages.StringField(3)
  updateMask = _messages.StringField(4)


class CloudnumberregistryProjectsLocationsListRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsListRequest object.

  Fields:
    extraLocationTypes: Optional. Do not use this field unless explicitly
      documented otherwise. This is primarily for internal usage.
    filter: A filter to narrow down results to a preferred subset. The
      filtering language accepts strings like `"displayName=tokyo"`, and is
      documented in more detail in [AIP-160](https://google.aip.dev/160).
    name: The resource that owns the locations collection, if applicable.
    pageSize: The maximum number of results to return. If not set, the service
      selects a default.
    pageToken: A page token received from the `next_page_token` field in the
      response. Send that page token to receive the subsequent page.
  """

  extraLocationTypes = _messages.StringField(1, repeated=True)
  filter = _messages.StringField(2)
  name = _messages.StringField(3, required=True)
  pageSize = _messages.IntegerField(4, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(5)


class CloudnumberregistryProjectsLocationsOperationsCancelRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsOperationsCancelRequest object.

  Fields:
    cancelOperationRequest: A CancelOperationRequest resource to be passed as
      the request body.
    name: The name of the operation resource to be cancelled.
  """

  cancelOperationRequest = _messages.MessageField('CancelOperationRequest', 1)
  name = _messages.StringField(2, required=True)


class CloudnumberregistryProjectsLocationsOperationsDeleteRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsOperationsDeleteRequest object.

  Fields:
    name: The name of the operation resource to be deleted.
  """

  name = _messages.StringField(1, required=True)


class CloudnumberregistryProjectsLocationsOperationsGetRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsOperationsGetRequest object.

  Fields:
    name: The name of the operation resource.
  """

  name = _messages.StringField(1, required=True)


class CloudnumberregistryProjectsLocationsOperationsListRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsOperationsListRequest object.

  Fields:
    filter: The standard list filter.
    name: The name of the operation's parent resource.
    pageSize: The standard list page size.
    pageToken: The standard list page token.
    returnPartialSuccess: When set to `true`, operations that are reachable
      are returned as normal, and those that are unreachable are returned in
      the ListOperationsResponse.unreachable field. This can only be `true`
      when reading across collections. For example, when `parent` is set to
      `"projects/example/locations/-"`. This field is not supported by default
      and will result in an `UNIMPLEMENTED` error if set unless explicitly
      documented otherwise in service or product specific documentation.
  """

  filter = _messages.StringField(1)
  name = _messages.StringField(2, required=True)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  returnPartialSuccess = _messages.BooleanField(5)


class CloudnumberregistryProjectsLocationsRealmsCreateRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsRealmsCreateRequest object.

  Fields:
    parent: Required. The parent resource name where the Realm will be
      created.
    realm: A Realm resource to be passed as the request body.
    realmId: Required. The ID to use for the Realm, which will become the
      final segment of the resource name.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request.
  """

  parent = _messages.StringField(1, required=True)
  realm = _messages.MessageField('Realm', 2)
  realmId = _messages.StringField(3)
  requestId = _messages.StringField(4)


class CloudnumberregistryProjectsLocationsRealmsDeleteRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsRealmsDeleteRequest object.

  Fields:
    force: Optional. If set to true, all associated resources will be deleted.
    name: Required. The resource name of the Realm to delete.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes after the first
      request.
  """

  force = _messages.BooleanField(1)
  name = _messages.StringField(2, required=True)
  requestId = _messages.StringField(3)


class CloudnumberregistryProjectsLocationsRealmsGetRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsRealmsGetRequest object.

  Enums:
    ViewValueValuesEnum: Optional. The view of the Realm to retrieve.

  Fields:
    name: Required. The resource name of the Realm to retrieve.
    view: Optional. The view of the Realm to retrieve.
  """

  class ViewValueValuesEnum(_messages.Enum):
    r"""Optional. The view of the Realm to retrieve.

    Values:
      REALM_VIEW_UNSPECIFIED: Unspecified view. Defaults to BASIC.
      BASIC: Basic view.
      FULL: Full view. Includes the same data as the BASIC view.
      AGGREGATE: Aggregate includes data about the child resources of the
        Realm.
    """
    REALM_VIEW_UNSPECIFIED = 0
    BASIC = 1
    FULL = 2
    AGGREGATE = 3

  name = _messages.StringField(1, required=True)
  view = _messages.EnumField('ViewValueValuesEnum', 2)


class CloudnumberregistryProjectsLocationsRealmsListRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsRealmsListRequest object.

  Enums:
    ViewValueValuesEnum: Optional. The view of the Realm to retrieve.

  Fields:
    filter: Optional. Filter expression to filter the results.
    orderBy: Optional. Hint for how to order the results.
    pageSize: Optional. Requested page size. Server may return fewer items
      than requested. If unspecified, server will pick an appropriate default.
    pageToken: Optional. A token identifying a page of results the server
      should return.
    parent: Required. The parent resource name, for example
      `projects/*/locations/*`.
    view: Optional. The view of the Realm to retrieve.
  """

  class ViewValueValuesEnum(_messages.Enum):
    r"""Optional. The view of the Realm to retrieve.

    Values:
      REALM_VIEW_UNSPECIFIED: Unspecified view. Defaults to BASIC.
      BASIC: Basic view.
      FULL: Full view. Includes the same data as the BASIC view.
      AGGREGATE: Aggregate includes data about the child resources of the
        Realm.
    """
    REALM_VIEW_UNSPECIFIED = 0
    BASIC = 1
    FULL = 2
    AGGREGATE = 3

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)
  view = _messages.EnumField('ViewValueValuesEnum', 6)


class CloudnumberregistryProjectsLocationsRealmsPatchRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsRealmsPatchRequest object.

  Fields:
    name: Required. Identifier. The resource name of the Realm.
    realm: A Realm resource to be passed as the request body.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request.
    updateMask: Optional. Field mask is used to specify the fields to be
      overwritten in the Realm resource by the update.
  """

  name = _messages.StringField(1, required=True)
  realm = _messages.MessageField('Realm', 2)
  requestId = _messages.StringField(3)
  updateMask = _messages.StringField(4)


class CloudnumberregistryProjectsLocationsRegistryBooksCreateRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsRegistryBooksCreateRequest object.

  Fields:
    parent: Required. The parent resource name where the RegistryBook will be
      created.
    registryBook: A RegistryBook resource to be passed as the request body.
    registryBookId: Required. The ID to use for the RegistryBook, which will
      become the final segment of the resource name.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  parent = _messages.StringField(1, required=True)
  registryBook = _messages.MessageField('RegistryBook', 2)
  registryBookId = _messages.StringField(3)
  requestId = _messages.StringField(4)


class CloudnumberregistryProjectsLocationsRegistryBooksDeleteRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsRegistryBooksDeleteRequest object.

  Fields:
    force: Optional. If set to true, all associated resources will be deleted.
    name: Required. The resource name of the RegistryBook to delete.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes after the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  force = _messages.BooleanField(1)
  name = _messages.StringField(2, required=True)
  requestId = _messages.StringField(3)


class CloudnumberregistryProjectsLocationsRegistryBooksGetRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsRegistryBooksGetRequest object.

  Enums:
    ViewValueValuesEnum: Optional. The view of the RegistryBook to retrieve.

  Fields:
    name: Required. The resource name of the RegistryBook to retrieve.
    view: Optional. The view of the RegistryBook to retrieve.
  """

  class ViewValueValuesEnum(_messages.Enum):
    r"""Optional. The view of the RegistryBook to retrieve.

    Values:
      REGISTRY_BOOK_VIEW_UNSPECIFIED: Unspecified view. Defaults to BASIC.
      BASIC: Basic view.
      FULL: Full view. Includes the same data as the BASIC view.
      AGGREGATE: Aggregate includes data about the child resources of the
        RegistryBook.
    """
    REGISTRY_BOOK_VIEW_UNSPECIFIED = 0
    BASIC = 1
    FULL = 2
    AGGREGATE = 3

  name = _messages.StringField(1, required=True)
  view = _messages.EnumField('ViewValueValuesEnum', 2)


class CloudnumberregistryProjectsLocationsRegistryBooksListRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsRegistryBooksListRequest object.

  Enums:
    ViewValueValuesEnum: Optional. The view of the RegistryBook to retrieve.

  Fields:
    filter: Optional. Filter expression to filter the results.
    orderBy: Optional. Hint for how to order the results.
    pageSize: Optional. Requested page size. Server may return fewer items
      than requested. If unspecified, server will pick an appropriate default.
    pageToken: Optional. A token identifying a page of results the server
      should return.
    parent: Required. The parent resource name, for example
      `projects/*/locations/*`.
    view: Optional. The view of the RegistryBook to retrieve.
  """

  class ViewValueValuesEnum(_messages.Enum):
    r"""Optional. The view of the RegistryBook to retrieve.

    Values:
      REGISTRY_BOOK_VIEW_UNSPECIFIED: Unspecified view. Defaults to BASIC.
      BASIC: Basic view.
      FULL: Full view. Includes the same data as the BASIC view.
      AGGREGATE: Aggregate includes data about the child resources of the
        RegistryBook.
    """
    REGISTRY_BOOK_VIEW_UNSPECIFIED = 0
    BASIC = 1
    FULL = 2
    AGGREGATE = 3

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)
  view = _messages.EnumField('ViewValueValuesEnum', 6)


class CloudnumberregistryProjectsLocationsRegistryBooksPatchRequest(_messages.Message):
  r"""A CloudnumberregistryProjectsLocationsRegistryBooksPatchRequest object.

  Fields:
    name: Required. Identifier. The resource name of the RegistryBook.
    registryBook: A RegistryBook resource to be passed as the request body.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
    updateMask: Optional. Field mask is used to specify the fields to be
      overwritten in the RegistryBook resource by the update. The fields
      specified in the update_mask are relative to the resource, not the full
      request. A field will be overwritten if it is in the mask. If the user
      does not provide a mask then all fields will be overwritten.
  """

  name = _messages.StringField(1, required=True)
  registryBook = _messages.MessageField('RegistryBook', 2)
  requestId = _messages.StringField(3)
  updateMask = _messages.StringField(4)


class CloudnumberregistryProjectsLocationsRegistryBooksSearchIpResourcesRequest(_messages.Message):
  r"""A
  CloudnumberregistryProjectsLocationsRegistryBooksSearchIpResourcesRequest
  object.

  Fields:
    name: Required. The resource name of the RegistryBook to search in.
    searchIpResourcesRequest: A SearchIpResourcesRequest resource to be passed
      as the request body.
  """

  name = _messages.StringField(1, required=True)
  searchIpResourcesRequest = _messages.MessageField('SearchIpResourcesRequest', 2)


class CustomRange(_messages.Message):
  r"""A CustomRange represents a user-defined IP address range.

  Messages:
    LabelsValue: Optional. User-defined labels.

  Fields:
    attributes: Optional. The attributes of the CustomRange.
    description: Optional. The description of the CustomRange.
    ipv4CidrRange: Optional. The IPv4 CIDR range of the CustomRange.
    ipv6CidrRange: Optional. The IPv6 CIDR range of the CustomRange.
    labels: Optional. User-defined labels.
    name: Required. Identifier. The resource name of the CustomRange, in the
      format
      `projects/{project}/locations/{location}/customRanges/{custom_range}`.
    parentRange: Optional. The resource name of the parent CustomRange, in the
      format
      `projects/{project}/locations/{location}/customRanges/{custom_range}`.
      If specified, the parent CustomRange must be in the same RegistryBook.
      This field is mutually exclusive with the `realm` field, as the Realm is
      inherited from the parent CustomRange.
    realm: Optional. The resource name of the Realm associated with the
      CustomRange, in the format
      `projects/{project}/locations/{location}/realms/{realm}`. The Realm must
      be in the same project as the CustomRange. This field must not be set if
      the `parent_range` field is set, as the Realm will be inherited from the
      parent CustomRange.
    registryBook: Output only. The RegistryBook of the CustomRange. This field
      is inherited from the Realm or parent CustomRange depending on which one
      is specified.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Optional. User-defined labels.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  attributes = _messages.MessageField('Attribute', 1, repeated=True)
  description = _messages.StringField(2)
  ipv4CidrRange = _messages.StringField(3)
  ipv6CidrRange = _messages.StringField(4)
  labels = _messages.MessageField('LabelsValue', 5)
  name = _messages.StringField(6)
  parentRange = _messages.StringField(7)
  realm = _messages.StringField(8)
  registryBook = _messages.StringField(9)


class DisableIpamAdminScopeRequest(_messages.Message):
  r"""Request message for the CloudNumberRegistry.DisableIpamAdminScope
  method.

  Fields:
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes after the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  requestId = _messages.StringField(1)


class DiscoveredRange(_messages.Message):
  r"""A DiscoveredRange represents an IP address range automatically detected
  by the discovery pipeline.

  Messages:
    LabelsValue: Optional. User-defined labels.

  Fields:
    attributes: Optional. The attributes of the DiscoveredRange.
    childCidrOverlapAllowed: Output only. If true, allows child
      DiscoveredRanges of this DiscoveredRange to overlap with each other.
    createTime: Output only. The time at which the DiscoveredRange was
      created.
    description: Optional. Description of the DiscoveredRange.
    discoveryMetadata: Output only. Owner metadata for this DiscoveredRange. A
      unique set of metadata is associated with each DiscoveredRange. If an IP
      range is shared by multiple resources (e.g., an Address resource and an
      Instance resource, or multiple ForwardingRules),separate
      DiscoveredRanges are created, each with a distinct owner. This ensures
      each DiscoveredRange has its own unique set of attributes, labels,
      create time and update time.
    ipv4CidrRange: Optional. The IPv4 CIDR range of the DiscoveredRange.
    ipv6CidrRange: Optional. The IPv6 CIDR range of the DiscoveredRange.
    labels: Optional. User-defined labels.
    name: Required. Identifier. The resource name of the DiscoveredRange, in
      the format `projects/{project}/locations/{location}/discoveredRanges/{di
      scovered_range}`.
    parentRange: Optional. The resource name of the parent DiscoveredRange, in
      the format `projects/{project}/locations/{location}/discoveredRanges/{di
      scovered_range}`.
    realm: Optional. The Realm of the DiscoveredRange.
    registryBook: Output only. The RegistryBook of the DiscoveredRange.
    updateTime: Output only. The time at which the DiscoveredRange was last
      updated.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Optional. User-defined labels.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  attributes = _messages.MessageField('Attribute', 1, repeated=True)
  childCidrOverlapAllowed = _messages.BooleanField(2)
  createTime = _messages.StringField(3)
  description = _messages.StringField(4)
  discoveryMetadata = _messages.MessageField('DiscoveryMetadata', 5)
  ipv4CidrRange = _messages.StringField(6)
  ipv6CidrRange = _messages.StringField(7)
  labels = _messages.MessageField('LabelsValue', 8)
  name = _messages.StringField(9)
  parentRange = _messages.StringField(10)
  realm = _messages.StringField(11)
  registryBook = _messages.StringField(12)
  updateTime = _messages.StringField(13)


class DiscoveryMetadata(_messages.Message):
  r"""Metadata about a discovered resource, tracking event times, state, and
  source information.

  Enums:
    StateValueValuesEnum: Output only. The state of the resource.

  Fields:
    createTime: Output only. The time when the resource was created.
    eventTime: Output only. The time when the event happened.
    resource: Output only. The resource name of the discovered resource,
      should be API-agnostic. Example:
      "projects/{project_number}/networks/{network_id}".
    resourceUri: Output only. The resource uri of the discovered resource.
    sourceId: Output only. The canonical google.aip.dev/122 name of the source
      resource.
    sourceSubId: Output only. A single source resource can be the source of
      multiple CNR resources. This sub_id is used to distinguish between the
      different CNR resources derived from the same upstream resource. For
      example, a single subnetwork can be the source of multiple Ranges, one
      for each protocol. In this case, the sub_id could be "private-ipv4" or
      "private-ipv6".
    state: Output only. The state of the resource.
    updateTime: Output only. The time when the resource was last modified.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. The state of the resource.

    Values:
      RESOURCE_STATE_UNSPECIFIED: Unspecified state.
      INVALID: The resource is in an invalid state.
      EXISTS: The resource exists in the backing store (is not tombstoned or
        completely missing) and there were no failures reading out the
        resource level metadata.
      DOES_NOT_EXIST: Resource does not exist or has been deleted or soft-
        deleted.
      ERROR: There was an error reading out the resource level metadata.
    """
    RESOURCE_STATE_UNSPECIFIED = 0
    INVALID = 1
    EXISTS = 2
    DOES_NOT_EXIST = 3
    ERROR = 4

  createTime = _messages.StringField(1)
  eventTime = _messages.StringField(2)
  resource = _messages.StringField(3)
  resourceUri = _messages.StringField(4)
  sourceId = _messages.StringField(5)
  sourceSubId = _messages.StringField(6)
  state = _messages.EnumField('StateValueValuesEnum', 7)
  updateTime = _messages.StringField(8)


class Empty(_messages.Message):
  r"""A generic empty message that you can re-use to avoid defining duplicated
  empty messages in your APIs. A typical example is to use it as the request
  or the response type of an API method. For instance: service Foo { rpc
  Bar(google.protobuf.Empty) returns (google.protobuf.Empty); }
  """



class FindCustomRangeFreeIpRangesResponse(_messages.Message):
  r"""Response message for the CloudNumberRegistry.FindCustomRangeFreeIpRanges
  method.

  Fields:
    freeIpCidrRanges: Output only. The free IP CIDR ranges found.
  """

  freeIpCidrRanges = _messages.StringField(1, repeated=True)


class FindDiscoveredRangeFreeIpRangesResponse(_messages.Message):
  r"""Response message for the
  CloudNumberRegistry.FindDiscoveredRangeFreeIpRanges method.

  Fields:
    freeIpCidrRanges: Output only. The free IP CIDR ranges found.
  """

  freeIpCidrRanges = _messages.StringField(1, repeated=True)


class IpamAdminScope(_messages.Message):
  r"""An IpamAdminScope defines the administrative boundary for IP address
  discovery and management. It configures which platforms and organizational
  scopes are enabled for IP tracking.

  Enums:
    EnabledAddonPlatformsValueListEntryValuesEnum:
    StateValueValuesEnum: Output only. State of resource discovery pipeline.

  Messages:
    LabelsValue: Optional. User-defined labels.

  Fields:
    createTime: Output only. The time at which the IpamAdminScope was created.
    enabledAddonPlatforms: Required. Add-on platforms that are enabled for
      this IpamAdminScope. Cloud Number Registry only discovers the IP
      addresses from the enabled platforms.
    labels: Optional. User-defined labels.
    name: Required. Identifier. The resource name of the IpamAdminScope.
    scopes: Required. Administrative scopes enabled for IP address discovery
      and management. For example, "organizations/1234567890". Minimum of 1
      scope is required. In preview, only one organization scope is allowed.
    state: Output only. State of resource discovery pipeline.
    updateTime: Output only. The time at which the IpamAdminScope was last
      updated.
  """

  class EnabledAddonPlatformsValueListEntryValuesEnum(_messages.Enum):
    r"""EnabledAddonPlatformsValueListEntryValuesEnum enum type.

    Values:
      ADD_ON_PLATFORM_UNSPECIFIED: Unspecified add-on platform.
      COMPUTE_ENGINE: Google Compute Engine.
      GCE: Deprecated: Use COMPUTE_ENGINE instead.
    """
    ADD_ON_PLATFORM_UNSPECIFIED = 0
    COMPUTE_ENGINE = 1
    GCE = 2

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. State of resource discovery pipeline.

    Values:
      DISCOVERY_PIPELINE_STATE_UNSPECIFIED: Unspecified state.
      INTERNAL_FAILURE: Internal failure.
      FAILED: Failure.
      SETUP_IN_PROGRESS: Setup in progress.
      READY_FOR_USE: Ready for use.
      DELETING_IN_PROGRESS: Deleting in progress.
      UPDATING: Updating.
      RECOVERING: Recovering.
      DISABLED: Disabled.
      DELETION_COMPLETED: Deleting completed.
      CLEANUP_IN_PROGRESS: Cleanup in progress.
      READY_FOR_DELETION: Ready for deletion.
    """
    DISCOVERY_PIPELINE_STATE_UNSPECIFIED = 0
    INTERNAL_FAILURE = 1
    FAILED = 2
    SETUP_IN_PROGRESS = 3
    READY_FOR_USE = 4
    DELETING_IN_PROGRESS = 5
    UPDATING = 6
    RECOVERING = 7
    DISABLED = 8
    DELETION_COMPLETED = 9
    CLEANUP_IN_PROGRESS = 10
    READY_FOR_DELETION = 11

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Optional. User-defined labels.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  createTime = _messages.StringField(1)
  enabledAddonPlatforms = _messages.EnumField('EnabledAddonPlatformsValueListEntryValuesEnum', 2, repeated=True)
  labels = _messages.MessageField('LabelsValue', 3)
  name = _messages.StringField(4)
  scopes = _messages.StringField(5, repeated=True)
  state = _messages.EnumField('StateValueValuesEnum', 6)
  updateTime = _messages.StringField(7)


class IpamAdminScopeAvailability(_messages.Message):
  r"""Availability details for a specific IpamAdminScope.

  Enums:
    AvailabilityValueValuesEnum: The availability of the scope.

  Fields:
    adminProject: The admin project of the IpamAdminScope if it exists.
    availability: The availability of the scope.
    scope: The scope of the IpamAdminScope.
  """

  class AvailabilityValueValuesEnum(_messages.Enum):
    r"""The availability of the scope.

    Values:
      AVAILABILITY_UNSPECIFIED: Unspecified availability.
      AVAILABLE: The scope is available.
      UNAVAILABLE: The scope is not available.
    """
    AVAILABILITY_UNSPECIFIED = 0
    AVAILABLE = 1
    UNAVAILABLE = 2

  adminProject = _messages.IntegerField(1)
  availability = _messages.EnumField('AvailabilityValueValuesEnum', 2)
  scope = _messages.StringField(3)


class ListCustomRangesResponse(_messages.Message):
  r"""Response message for the CloudNumberRegistry.ListCustomRanges method.

  Fields:
    customRanges: The list of CustomRanges.
    nextPageToken: A token to retrieve the next page of results, or empty if
      there are no more results in the list.
    unreachable: Locations that could not be reached.
  """

  customRanges = _messages.MessageField('CustomRange', 1, repeated=True)
  nextPageToken = _messages.StringField(2)
  unreachable = _messages.StringField(3, repeated=True)


class ListDiscoveredRangesResponse(_messages.Message):
  r"""Response message for the CloudNumberRegistry.ListDiscoveredRanges
  method.

  Fields:
    discoveredRanges: The list of DiscoveredRanges.
    nextPageToken: A token to retrieve the next page of results, or empty if
      there are no more results in the list.
    unreachable: Locations that could not be reached.
  """

  discoveredRanges = _messages.MessageField('DiscoveredRange', 1, repeated=True)
  nextPageToken = _messages.StringField(2)
  unreachable = _messages.StringField(3, repeated=True)


class ListIpamAdminScopesResponse(_messages.Message):
  r"""Response message for the CloudNumberRegistry.ListIpamAdminScopes method.

  Fields:
    ipamAdminScopes: The list of IpamAdminScopes.
    nextPageToken: A token to retrieve the next page of results, or empty if
      there are no more results in the list.
    unreachable: Locations that could not be reached.
  """

  ipamAdminScopes = _messages.MessageField('IpamAdminScope', 1, repeated=True)
  nextPageToken = _messages.StringField(2)
  unreachable = _messages.StringField(3, repeated=True)


class ListLocationsResponse(_messages.Message):
  r"""The response message for Locations.ListLocations.

  Fields:
    locations: A list of locations that matches the specified filter in the
      request.
    nextPageToken: The standard List next-page token.
  """

  locations = _messages.MessageField('Location', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListOperationsResponse(_messages.Message):
  r"""The response message for Operations.ListOperations.

  Fields:
    nextPageToken: The standard List next-page token.
    operations: A list of operations that matches the specified filter in the
      request.
    unreachable: Unordered list. Unreachable resources. Populated when the
      request sets `ListOperationsRequest.return_partial_success` and reads
      across collections. For example, when attempting to list all resources
      across all supported locations.
  """

  nextPageToken = _messages.StringField(1)
  operations = _messages.MessageField('Operation', 2, repeated=True)
  unreachable = _messages.StringField(3, repeated=True)


class ListRealmsResponse(_messages.Message):
  r"""Response message for the CloudNumberRegistry.ListRealms method.

  Fields:
    nextPageToken: A token identifying a page of results the server should
      return.
    realms: The list of Realms.
    unreachable: Locations that could not be reached.
  """

  nextPageToken = _messages.StringField(1)
  realms = _messages.MessageField('Realm', 2, repeated=True)
  unreachable = _messages.StringField(3, repeated=True)


class ListRegistryBooksResponse(_messages.Message):
  r"""Response message for the CloudNumberRegistry.ListRegistryBooks method.

  Fields:
    nextPageToken: A token identifying a page of results the server should
      return.
    registryBooks: The list of RegistryBook
    unreachable: Locations that could not be reached.
  """

  nextPageToken = _messages.StringField(1)
  registryBooks = _messages.MessageField('RegistryBook', 2, repeated=True)
  unreachable = _messages.StringField(3, repeated=True)


class Location(_messages.Message):
  r"""A resource that represents a Google Cloud location.

  Messages:
    LabelsValue: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    MetadataValue: Service-specific metadata. For example the available
      capacity at the given location.

  Fields:
    displayName: The friendly name for this location, typically a nearby city
      name. For example, "Tokyo".
    labels: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    locationId: The canonical id for this location. For example: `"us-east1"`.
    metadata: Service-specific metadata. For example the available capacity at
      the given location.
    name: Resource name for the location, which may vary between
      implementations. For example: `"projects/example-project/locations/us-
      east1"`
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Cross-service attributes for the location. For example
    {"cloud.googleapis.com/region": "us-east1"}

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata. For example the available capacity at the
    given location.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  displayName = _messages.StringField(1)
  labels = _messages.MessageField('LabelsValue', 2)
  locationId = _messages.StringField(3)
  metadata = _messages.MessageField('MetadataValue', 4)
  name = _messages.StringField(5)


class Operation(_messages.Message):
  r"""This resource represents a long-running operation that is the result of
  a network API call.

  Messages:
    MetadataValue: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    ResponseValue: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

  Fields:
    done: If the value is `false`, it means the operation is still in
      progress. If `true`, the operation is completed, and either `error` or
      `response` is available.
    error: The error result of the operation in case of failure or
      cancellation.
    metadata: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    name: The server-assigned name, which is only unique within the same
      service that originally returns it. If you use the default HTTP mapping,
      the `name` should be a resource name ending with
      `operations/{unique_id}`.
    response: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata associated with the operation. It typically
    contains progress information and common metadata such as create time.
    Some services might not provide such metadata. Any method that returns a
    long-running operation should document the metadata type, if any.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ResponseValue(_messages.Message):
    r"""The normal, successful response of the operation. If the original
    method returns no data on success, such as `Delete`, the response is
    `google.protobuf.Empty`. If the original method is standard
    `Get`/`Create`/`Update`, the response should be the resource. For other
    methods, the response should have the type `XxxResponse`, where `Xxx` is
    the original method name. For example, if the original method name is
    `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

    Messages:
      AdditionalProperty: An additional property for a ResponseValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ResponseValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  done = _messages.BooleanField(1)
  error = _messages.MessageField('Status', 2)
  metadata = _messages.MessageField('MetadataValue', 3)
  name = _messages.StringField(4)
  response = _messages.MessageField('ResponseValue', 5)


class OperationMetadata(_messages.Message):
  r"""Represents the metadata of the long-running operation.

  Fields:
    apiVersion: Output only. API version used to start the operation.
    createTime: Output only. The time the operation was created.
    endTime: Output only. The time the operation finished running.
    requestedCancellation: Output only. Identifies whether the user has
      requested cancellation of the operation. Operations that have been
      cancelled successfully have Operation.error value with a
      google.rpc.Status.code of 1, corresponding to `Code.CANCELLED`.
    statusMessage: Output only. Human-readable status of the operation, if
      any.
    target: Output only. Server-defined resource path for the target of the
      operation.
    verb: Output only. Name of the verb executed by the operation.
  """

  apiVersion = _messages.StringField(1)
  createTime = _messages.StringField(2)
  endTime = _messages.StringField(3)
  requestedCancellation = _messages.BooleanField(4)
  statusMessage = _messages.StringField(5)
  target = _messages.StringField(6)
  verb = _messages.StringField(7)


class Range(_messages.Message):
  r"""Represents either a CustomRange or a DiscoveredRange.

  Fields:
    customRange: A CustomRange.
    discoveredRange: A DiscoveredRange.
    utilization: The utilization of the Range.
  """

  customRange = _messages.MessageField('CustomRange', 1)
  discoveredRange = _messages.MessageField('DiscoveredRange', 2)
  utilization = _messages.MessageField('RangeUtilization', 3)


class RangeUtilization(_messages.Message):
  r"""Utilization metrics for an IP Range, including consumed and produced
  address counts.

  Fields:
    totalConsumed: Output only. The total number of IP addresses consumed in
      the Range.
    totalProduced: Output only. The total number of IP addresses produced in
      the Range.
    usage: Output only. The usage of the Range as a percentage. This is marked
      as optional so that we have presence tracking and API responses show 0.0
      instead of NULL.
  """

  totalConsumed = _messages.StringField(1)
  totalProduced = _messages.StringField(2)
  usage = _messages.FloatField(3)


class Realm(_messages.Message):
  r"""A Realm represents a distinct network domain or security zone. It groups
  Ranges that share the same traffic and management characteristics. All the
  ranges in a Realm are routable to each other, meaning that they cannot
  overlap.

  Enums:
    IpVersionValueValuesEnum: Optional. IP version of the Realm.
    ManagementTypeValueValuesEnum: Required. Management type of the Realm.
    TrafficTypeValueValuesEnum: Required. Traffic type of the Realm.

  Messages:
    LabelsValue: Optional. User-defined labels.

  Fields:
    aggregatedData: Output only. Aggregated data for the Realm. Populated only
      when the view is AGGREGATE.
    createTime: Output only. The time at which the Realm was created.
    discoveryMetadata: Output only. Discovery metadata of the Realm.
    ipVersion: Optional. IP version of the Realm.
    labels: Optional. User-defined labels.
    managementType: Required. Management type of the Realm.
    name: Required. Identifier. The resource name of the Realm.
    registryBook: Required. Name of the RegistryBook that claims the Realm.
    trafficType: Required. Traffic type of the Realm.
    updateTime: Output only. The time at which the Realm was last updated.
  """

  class IpVersionValueValuesEnum(_messages.Enum):
    r"""Optional. IP version of the Realm.

    Values:
      IP_VERSION_UNSPECIFIED: Unspecified IP version.
      IPV4: IPv4 IP version.
      IPV6: IPv6 IP version.
    """
    IP_VERSION_UNSPECIFIED = 0
    IPV4 = 1
    IPV6 = 2

  class ManagementTypeValueValuesEnum(_messages.Enum):
    r"""Required. Management type of the Realm.

    Values:
      MANAGEMENT_TYPE_UNSPECIFIED: Unspecified management type.
      CNR: Managed by Cloud Number Registry.
      USER: Managed by User.
    """
    MANAGEMENT_TYPE_UNSPECIFIED = 0
    CNR = 1
    USER = 2

  class TrafficTypeValueValuesEnum(_messages.Enum):
    r"""Required. Traffic type of the Realm.

    Values:
      TRAFFIC_TYPE_UNSPECIFIED: Unspecified traffic type.
      UNSET: Unset traffic type.
      INTERNET: Internet traffic.
      PRIVATE: Private traffic.
      LINKLOCAL: Linklocal traffic.
    """
    TRAFFIC_TYPE_UNSPECIFIED = 0
    UNSET = 1
    INTERNET = 2
    PRIVATE = 3
    LINKLOCAL = 4

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Optional. User-defined labels.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  aggregatedData = _messages.MessageField('RealmAggregatedData', 1)
  createTime = _messages.StringField(2)
  discoveryMetadata = _messages.MessageField('DiscoveryMetadata', 3)
  ipVersion = _messages.EnumField('IpVersionValueValuesEnum', 4)
  labels = _messages.MessageField('LabelsValue', 5)
  managementType = _messages.EnumField('ManagementTypeValueValuesEnum', 6)
  name = _messages.StringField(7)
  registryBook = _messages.StringField(8)
  trafficType = _messages.EnumField('TrafficTypeValueValuesEnum', 9)
  updateTime = _messages.StringField(10)


class RealmAggregatedData(_messages.Message):
  r"""Aggregated data for the Realm.

  Fields:
    customRangesCount: Output only. Number of CustomRanges in the Realm.
    discoveredRangesCount: Output only. Number of DiscoveredRanges in the
      Realm.
  """

  customRangesCount = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  discoveredRangesCount = _messages.IntegerField(2, variant=_messages.Variant.INT32)


class RegistryBook(_messages.Message):
  r"""A RegistryBook organizes and manages IP address space. It claims
  specific scopes (such as projects) and groups related Realms and Ranges.

  Messages:
    LabelsValue: Optional. User-defined labels.

  Fields:
    aggregatedData: Output only. Aggregated data for the RegistryBook.
      Populated only when the view is AGGREGATE.
    claimedScopes: Optional. List of scopes claimed by the RegistryBook. In
      Preview, Only project scope is supported. Each scope is in the format of
      projects/{project}. Each scope can only be claimed once.
    createTime: Output only. The time at which the RegistryBook was created.
    isDefault: Output only. Whether the RegistryBook is the default one.
    labels: Optional. User-defined labels.
    name: Required. Identifier. The resource name of the RegistryBook.
    updateTime: Output only. The time at which the RegistryBook was last
      updated.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Optional. User-defined labels.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  aggregatedData = _messages.MessageField('AggregatedData', 1)
  claimedScopes = _messages.StringField(2, repeated=True)
  createTime = _messages.StringField(3)
  isDefault = _messages.BooleanField(4)
  labels = _messages.MessageField('LabelsValue', 5)
  name = _messages.StringField(6)
  updateTime = _messages.StringField(7)


class SearchIpResourcesRequest(_messages.Message):
  r"""Request message for the CloudNumberRegistry.SearchIpResources method.

  Enums:
    SearchResourceTypesValueListEntryValuesEnum:

  Fields:
    orderBy: Optional. Hint for how to order the results. Supported sort
      fields are: - `name`: Sort alphabetically by the resource name. -
      `create_time`: Sort by the creation timestamp of the resource. -
      `update_time`: Sort by the last update timestamp of the resource.
      Supported directions are `asc` (ascending) and `desc` (descending). If
      unspecified, direction defaults to `asc`. Only sorting by a single field
      is supported.
    pageSize: Optional. Requested page size. Server may return fewer items
      than requested. If unspecified, server will pick an appropriate default.
    pageToken: Optional. A token identifying a page of results the server
      should return.
    query: Optional. Search query. This string filters resources in an
      AIP-160-like format. It has some limitations. You can only specify top
      level conjunctions or attribute level negations. Each restriction can
      only be used once except the attribute restriction. The available
      restrictions for Ranges are: - `realm`: The Realm name to search in. -
      `ip_address`: The IP address to search for within Ranges. -
      `ip_version`: The IP version to filter by (e.g., "IPV4", "IPV6"). -
      `parent_range`: The parent Range of the Range to search for. -
      `attribute_text`: The attribute text to search for within Ranges. -
      `attribute`: The attribute key and value to filter by. The available
      restrictions for Realms are: - `ip_version`: The IP version to search
      for. - `management_type`: The management type of the Realm (e.g., "CNR",
      "USER"). Only one of attribute_text or multiple attribute filters can be
      specified. Examples: - `realm=test-realm` - `realm=test-realm AND
      ip_address=10.0.0.0` - `realm=test-realm AND ip_version=IPV6` -
      `realm=test-realm AND attribute_text=test` - `ip_address=10.0.0.0 AND
      attribute:(key1=value1) AND attribute:(key2=value2)` -
      `attribute_text=test AND
      parent_range=projects/123/locations/global/discoveredRanges/test-parent-
      range` - `management_type=CNR`
    searchResourceTypes: Optional. The type of resources to search for. If not
      specified, the server will return Ranges.
    showUtilization: Optional. Whether to show the utilization of the Ranges
      in the response.
  """

  class SearchResourceTypesValueListEntryValuesEnum(_messages.Enum):
    r"""SearchResourceTypesValueListEntryValuesEnum enum type.

    Values:
      SEARCH_RESOURCE_TYPE_UNSPECIFIED: Unspecified search type.
      RANGES: Search for Ranges.
      REALMS: Search for Realms.
    """
    SEARCH_RESOURCE_TYPE_UNSPECIFIED = 0
    RANGES = 1
    REALMS = 2

  orderBy = _messages.StringField(1)
  pageSize = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(3)
  query = _messages.StringField(4)
  searchResourceTypes = _messages.EnumField('SearchResourceTypesValueListEntryValuesEnum', 5, repeated=True)
  showUtilization = _messages.BooleanField(6)


class SearchIpResourcesResponse(_messages.Message):
  r"""Response message for the CloudNumberRegistry.SearchIpResources method.

  Fields:
    nextPageToken: A token identifying a page of results the server should
      return.
    ranges: Deprecated: Use results field instead. The list of Ranges matching
      the search query.
    results: The list of results matching the search query.
    unreachable: Locations that could not be reached.
  """

  nextPageToken = _messages.StringField(1)
  ranges = _messages.MessageField('Range', 2, repeated=True)
  results = _messages.MessageField('SearchIpResourcesResult', 3, repeated=True)
  unreachable = _messages.StringField(4, repeated=True)


class SearchIpResourcesResult(_messages.Message):
  r"""A result matching the search query, which can be either a Range or a
  Realm.

  Fields:
    range: A Range matching the search query.
    realm: A Realm matching the search query.
  """

  range = _messages.MessageField('Range', 1)
  realm = _messages.MessageField('Realm', 2)


class ShowCustomRangeUtilizationResponse(_messages.Message):
  r"""Response message for the CloudNumberRegistry.ShowCustomRangeUtilization
  method.

  Fields:
    customRange: The CustomRange resource.
    rangeUtilization: The utilization details of the CustomRange.
  """

  customRange = _messages.MessageField('CustomRange', 1)
  rangeUtilization = _messages.MessageField('RangeUtilization', 2)


class ShowDiscoveredRangeUtilizationResponse(_messages.Message):
  r"""Response message for the
  CloudNumberRegistry.ShowDiscoveredRangeUtilization method.

  Fields:
    discoveredRange: The DiscoveredRange resource.
    rangeUtilization: The utilization details of the DiscoveredRange.
  """

  discoveredRange = _messages.MessageField('DiscoveredRange', 1)
  rangeUtilization = _messages.MessageField('RangeUtilization', 2)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


class Status(_messages.Message):
  r"""The `Status` type defines a logical error model that is suitable for
  different programming environments, including REST APIs and RPC APIs. It is
  used by [gRPC](https://github.com/grpc). Each `Status` message contains
  three pieces of data: error code, error message, and error details. You can
  find out more about this error model and how to work with it in the [API
  Design Guide](https://cloud.google.com/apis/design/errors).

  Messages:
    DetailsValueListEntry: A DetailsValueListEntry object.

  Fields:
    code: The status code, which should be an enum value of google.rpc.Code.
    details: A list of messages that carry the error details. There is a
      common set of message types for APIs to use.
    message: A developer-facing error message, which should be in English. Any
      user-facing error message should be localized and sent in the
      google.rpc.Status.details field, or localized by the client.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DetailsValueListEntry(_messages.Message):
    r"""A DetailsValueListEntry object.

    Messages:
      AdditionalProperty: An additional property for a DetailsValueListEntry
        object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DetailsValueListEntry object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  code = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  details = _messages.MessageField('DetailsValueListEntry', 2, repeated=True)
  message = _messages.StringField(3)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to test connection profiles for a database migration."""


from typing import Any

from googlecloudsdk.api_lib.database_migration import api_util
from googlecloudsdk.api_lib.database_migration import connection_profiles
from googlecloudsdk.api_lib.database_migration import resource_args
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.database_migration.connection_profiles import flags as cp_flags
from googlecloudsdk.core import log


DETAILED_HELP = {
    'DESCRIPTION': """
        Validates a Database Migration Service connection profile.
        """,
    'EXAMPLES': """\
        To test a connection profile:

            $ {command} my-profile --region=us-central1
        """,
}


@base.RegionalEndpointsSupported
@base.ReleaseTracks(base.ReleaseTrack.GA)
@base.DefaultUniverseOnly
class Test(base.Command):
  """Test a Database Migration Service connection profile."""

  detailed_help = DETAILED_HELP

  @staticmethod
  def Args(parser) -> None:
    """Args is called by calliope to gather arguments for this command.

    Args:
      parser: An argparse parser that you can use to add arguments that go on
        the command line after this command. Positional arguments are allowed.
    """
    resource_args.AddConnectionProfileResourceArg(parser, 'to test')
    cp_flags.AddNoAsyncFlag(parser)

  def Run(self, args: Any) -> Any:
    """Test a Database Migration Service connection profile.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
        with.

    Returns:
      A dict object representing the operations resource describing the test
      operation.
    """
    connection_profile_ref = args.CONCEPTS.connection_profile.Parse()
    location = connection_profile_ref.locationsId

    cp_client = connection_profiles.ConnectionProfilesClient(
        self.ReleaseTrack(),
        location=location,
    )
    result_operation = cp_client.Test(connection_profile_ref.RelativeName())

    client = api_util.GetClientInstance(self.ReleaseTrack(), location=location)
    messages = api_util.GetMessagesModule(self.ReleaseTrack())
    resource_parser = api_util.GetResourceParser(self.ReleaseTrack())

    if args.IsKnownAndSpecified('no_async'):
      log.status.Print(
          'Waiting for connection profile [{}] to be test with [{}]'.format(
              connection_profile_ref.connectionProfilesId, result_operation.name
          )
      )

      api_util.HandleLRO(
          client, result_operation, client.projects_locations_connectionProfiles
      )

      log.status.Print(
          'Tested connection profile {} [{}]'.format(
              connection_profile_ref.connectionProfilesId, result_operation.name
          )
      )
      return

    operation_ref = resource_parser.Create(
        'datamigration.projects.locations.operations',
        operationsId=result_operation.name,
        projectsId=connection_profile_ref.projectsId,
        locationsId=connection_profile_ref.locationsId,
    )

    return client.projects_locations_operations.Get(
        messages.DatamigrationProjectsLocationsOperationsGetRequest(
            name=operation_ref.operationsId
        )
    )



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for creating snapshot groups."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute.operations import poller
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import flags
from googlecloudsdk.command_lib.compute.snapshot_groups import flags as snap_flags
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources


def _CommonArgs(parser):
  """Add arguments used for parsing in all command tracks."""
  parser.add_argument('name', help='The name of the snapshot group.')
  snap_flags.SOURCE_INSTANT_SNAPSHOT_GROUP_ARG.AddArgument(parser)
  base.ASYNC_FLAG.AddToParser(parser)


def _RunCreate(self, args):
  """Shared logic for creating a snapshot group."""
  holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
  client = holder.client.apitools_client
  messages = holder.client.messages
  sg_ref = holder.resources.Parse(
      args.name,
      params={
          'project': properties.VALUES.core.project.GetOrFail,
      },
      collection='compute.snapshotGroups',
  )

  sg_message = messages.SnapshotGroup(
      name=sg_ref.Name()
  )

  isg_ref = snap_flags.SOURCE_INSTANT_SNAPSHOT_GROUP_ARG.ResolveAsResource(
      args,
      holder.resources,
      scope_lister=flags.GetDefaultScopeLister(holder.client),
  )
  sg_message.sourceInstantSnapshotGroup = isg_ref.SelfLink()

  request = messages.ComputeSnapshotGroupsInsertRequest(
      snapshotGroup=sg_message, project=sg_ref.project
  )
  result = client.snapshotGroups.Insert(request)
  operation_ref = resources.REGISTRY.Parse(
      result.name,
      params={'project': sg_ref.project},
      collection='compute.globalOperations',
  )
  if args.async_:
    log.UpdatedResource(
        operation_ref,
        kind='gce snapshot group {0}'.format(sg_ref.Name()),
        is_async=True,
        details=(
            'Use [gcloud compute operations describe] command '
            'to check the status of this operation.'
        ),
    )
    return result
  operation_poller = poller.Poller(client.snapshotGroups, sg_ref)
  return waiter.WaitFor(
      operation_poller,
      operation_ref,
      'Creating gce snapshot group {0}'.format(sg_ref.Name()),
  )


DETAILED_HELP = {
    'EXAMPLES': """\
        To create a Compute Engine snapshot group named 'my-snapshot-group'
        from an instant snapshot group 'my-isg' in zone 'us-central1-a', run:

          $ {command} my-snapshot-group \\
              --source-instant-snapshot-group=my-isg \\
              --source-instant-snapshot-group-zone=us-central1-a
        """,
}


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@base.DefaultUniverseOnly
class CreateAlpha(base.Command):
  """Create a Compute Engine snapshot group."""

  detailed_help = DETAILED_HELP

  @staticmethod
  def Args(parser):
    _CommonArgs(parser)

  def Run(self, args):
    return _RunCreate(self, args)


@base.Hidden  # Hide this command from public documentation
@base.ReleaseTracks(base.ReleaseTrack.BETA)
@base.DefaultUniverseOnly
class CreateBeta(base.Command):
  """Create a Compute Engine snapshot group."""

  detailed_help = DETAILED_HELP

  @staticmethod
  def Args(parser):
    _CommonArgs(parser)

  def Run(self, args):
    return _RunCreate(self, args)



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Flags and helpers for compute instances bulk commands."""


import re

from googlecloudsdk.api_lib.compute import constants
from googlecloudsdk.api_lib.compute import metadata_utils
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.compute import completers as compute_completers
from googlecloudsdk.command_lib.compute import flags as compute_flags
from googlecloudsdk.command_lib.compute.instances import flags as instances_flags
from googlecloudsdk.command_lib.compute.resource_policies import flags as maintenance_flags
from googlecloudsdk.command_lib.util.args import labels_util


def AddDiskArgsForBulk(parser):
  """Adds arguments related to disks for bulk insert."""

  disk_device_name_help = instances_flags.GetDiskDeviceNameHelp(
      container_mount_enabled=False
  )
  instances_flags.AddBootDiskArgs(parser, enable_kms=True)

  disk_arg_spec = {
      'name': str,
      'boot': arg_parsers.ArgBoolean(),
      'device-name': str,
      'scope': str,
  }

  disk_help = """
      Attaches persistent disks to the instances. The disks
      specified must already exist.

      *name*::: The disk to attach to the instances.

      *boot*::: If ``yes'', indicates that this is a boot disk. The
      virtual machines will use the first partition of the disk for
      their root file systems. The default value for this is ``no''.

      *device-name*::: {}

      *scope*::: Can be `zonal` or `regional`. If ``zonal'', the disk is
      interpreted as a zonal disk in the same zone as the instance (default).
      If ``regional'', the disk is interpreted as a regional disk in the same
      region as the instance. The default value for this is ``zonal''.
      """.format(disk_device_name_help)

  parser.add_argument(
      '--disk',
      type=arg_parsers.ArgDict(spec=disk_arg_spec),
      action='append',
      help=disk_help,
  )


def ValidateBulkDiskFlags(args):
  """Validates the values of all disk-related flags."""
  for disk in args.disk or []:
    if 'name' not in disk:
      raise exceptions.InvalidArgumentException(
          '--disk',
          '[name] is missing in [--disk]. [--disk] value must be of the form '
          '[{0}].'.format(instances_flags.DISK_METAVAR),
      )

  instances_flags.ValidateDiskBootFlags(args, enable_kms=True)
  instances_flags.ValidateCreateDiskFlags(
      args,
      enable_snapshots=True,
      enable_image_csek=True,
  )


def MakeBulkSourceInstanceTemplateArg():
  return compute_flags.ResourceArgument(
      name='--source-instance-template',
      resource_name='instance template',
      completer=compute_completers.InstanceTemplatesCompleter,
      required=False,
      global_collection='compute.instanceTemplates',
      short_help=(
          'The name of the instance template that the instance will '
          'be created from. Users can override fields by specifying '
          'other flags.'
      ),
  )


def AddDistributionTargetShapeArgs(parser):
  """Adds bulk creation target shape arguments to parser."""
  choices_text = {
      'ANY_SINGLE_ZONE': (
          'Enforces VM placement in one allowed zone. Use this to avoid '
          'cross-zone network egress or to reduce network latency. This is the '
          'default value.'
      ),
      'BALANCED': (
          'Allows distribution of VMs in zones where resources are available '
          'while distributing VMs as evenly as possible across selected zones '
          'to minimize the impact of zonal failures. Recommended for highly '
          'available serving or batch workloads.'
      ),
      'ANY': (
          'Allows creating VMs in multiple zones if one zone cannot '
          'accommodate all the requested VMs. The resulting distribution '
          'shapes can vary.'
      ),
  }
  parser.add_argument(
      '--target-distribution-shape',
      metavar='SHAPE',
      type=lambda x: x.upper(),
      choices=choices_text,
      help="""
        Specifies whether and how to distribute VMs across multiple zones in a
        region or to enforce placement of VMs in a single zone.
        The default shape is `ANY_SINGLE_ZONE`.
      """,
  )


def AddBulkCreateArgs(
    parser,
    add_zone_region_flags,
    support_max_count_per_zone,
    support_custom_hostnames,
):
  """Adds bulk creation specific arguments to parser."""
  parser.add_argument(
      '--count',
      type=int,
      help="""
      Number of Compute Engine virtual machines to create. If specified, and
      `--predefined-names` is specified, count must equal the amount of names
      provided to `--predefined-names`. If not specified,
      the number of virtual machines created will equal the number of names
      provided to `--predefined-names`.
    """,
  )
  parser.add_argument(
      '--min-count',
      type=int,
      help="""
        The minimum number of Compute Engine virtual machines that must be
        successfully created for the operation to be considered a success. If
        the operation successfully creates as many virtual machines as
        specified here they will be persisted, otherwise the operation rolls
        back and deletes all created virtual machines. If not specified, this
        value is equal to `--count`.""",
  )

  name_group = parser.add_group(mutex=True, required=True)
  name_group.add_argument(
      '--predefined-names',
      type=arg_parsers.ArgList(),
      metavar='INSTANCE_NAME',
      help="""
        List of predefined names for the Compute Engine virtual machines being
        created. If `--count` is specified alongside this flag, provided count
        must equal the amount of names provided to this flag. If `--count` is
        not specified, the number of virtual machines
        created will equal the number of names provided.
      """,
  )
  name_group.add_argument(
      '--name-pattern',
      help="""
        Name pattern for generating instance names. Specify a pattern with a
        single sequence of hash (#) characters that will be replaced with
        generated sequential numbers of instances. E.g. name pattern of
        'instance-###' will generate instance names 'instance-001',
        'instance-002', and so on, until the number of virtual machines
        specified using `--count` is reached. If instances matching name pattern
        exist, the new instances will be assigned names to avoid clashing with
        the existing ones. E.g. if there exists `instance-123`, the new
        instances will start at `instance-124` and increment from there.
      """,
  )
  if add_zone_region_flags:
    location = parser.add_group(required=True, mutex=True)
    location.add_argument(
        '--region',
        help="""
        Region in which to create the Compute Engine virtual machines. Compute
        Engine will select a zone in which to create all virtual machines.
    """,
    )
    location.add_argument(
        '--zone',
        help="""
        Zone in which to create the Compute Engine virtual machines.

        A list of zones can be fetched by running:

            $ gcloud compute zones list

        To unset the property, run:

            $ gcloud config unset compute/zone

        Alternatively, the zone can be stored in the environment variable
        CLOUDSDK_COMPUTE_ZONE.
     """,
    )
  parser.add_argument(
      '--location-policy',
      metavar='ZONE=POLICY',
      type=arg_parsers.ArgDict(),
      help="""
        Policy for which zones to include or exclude during bulk instance creation
        within a region. Policy is defined as a list of key-value pairs, with the
        key being the zone name, and value being the applied policy. Available
        policies are `allow` and `deny`. Default for zones if left unspecified is `allow`.

        Example:

          gcloud compute instances bulk create --name-pattern=example-###
            --count=5 --region=us-east1
            --location-policy=us-east1-b=allow,us-east1-c=deny
      """,
  )
  if support_max_count_per_zone:
    parser.add_argument(
        '--max-count-per-zone',
        metavar='ZONE=MAX_COUNT_PER_ZONE',
        type=arg_parsers.ArgDict(),
        help="""
          Maximum number of instances per zone specified as key-value pairs. The zone name is the key and the max count per zone
          is the value in that zone.

          Example:

            gcloud compute instances bulk create --name-pattern=example-###
              --count=5 --region=us-east1
              --max-count-per-zone=us-east1-b=2,us-east-1-c=1
        """,
    )
  if support_custom_hostnames:
    parser.add_argument(
        '--per-instance-hostnames',
        metavar='INSTANCE_NAME=INSTANCE_HOSTNAME',
        type=arg_parsers.ArgDict(key_type=str, value_type=str),
        help="""
          Specify the hostname of the instance to be created. The specified
          hostname must be RFC1035 compliant. If hostname is not specified, the
          default hostname is [INSTANCE_NAME].c.[PROJECT_ID].internal when using
          the global DNS, and [INSTANCE_NAME].[ZONE].c.[PROJECT_ID].internal
          when using zonal DNS.
        """,
    )


def AddBulkCreateNetworkingArgs(
    parser,
    support_igmp_query=False,
):
  """Adds Networking Args for Bulk Create Command."""

  multiple_network_interface_cards_spec = {
      'network': str,
      'subnet': str,
  }

  def ValidateNetworkTier(network_tier_input):
    network_tier = network_tier_input.upper()
    if network_tier in constants.NETWORK_TIER_CHOICES_FOR_INSTANCE:
      return network_tier
    else:
      raise exceptions.InvalidArgumentException(
          '--network-interface', 'Invalid value for network-tier'
      )

  multiple_network_interface_cards_spec['network-tier'] = ValidateNetworkTier
  multiple_network_interface_cards_spec['nic-type'] = (
      instances_flags.ValidateNetworkInterfaceNicType
  )

  network_interface_help = """
      Adds a network interface to the instance. Mutually exclusive with any
      of these flags: *--network*, *--network-tier*, *--no-address*, *--subnet*,
      *--stack-type*. This flag can be repeated to specify multiple network
      interfaces.

      *network*::: Specifies the network that the interface will be part of.
      If subnet is also specified it must be subnetwork of this network. If
      neither is specified, this defaults to the "default" network.

      *network-tier*::: Specifies the network tier of the interface.
      ``NETWORK_TIER'' must be one of: `PREMIUM`, `STANDARD`. The default
      value is `PREMIUM`.

      *subnet*::: Specifies the subnet that the interface will be part of.
      If network key is also specified this must be a subnetwork of the
      specified network.

      *nic-type*::: Specifies the  Network Interface Controller (NIC) type for
      the interface. ``NIC_TYPE'' must be one of: `GVNIC`, `VIRTIO_NET`.
  """

  multiple_network_interface_cards_spec['no-address'] = None
  network_interface_help += """
    *no-address*::: If specified the interface will have no external IP.
    If not specified instances will get ephemeral IPs.
    """

  multiple_network_interface_cards_spec['queue-count'] = int
  network_interface_help += """
    *queue-count*::: Specifies the networking queue count for this interface.
    Both Rx and Tx queues will be set to this number. If it's not specified, a
    default queue count will be assigned. See
    https://cloud.google.com/compute/docs/network-bandwidth#rx-tx for
    more details.
  """

  multiple_network_interface_cards_spec['stack-type'] = (
      instances_flags.ValidateNetworkInterfaceStackType
  )
  stack_types = '`IPV4_ONLY`, `IPV4_IPV6`, `IPV6_ONLY`'
  network_interface_help += f"""
    *stack-type*::: Specifies whether IPv6 is enabled on the interface.
    ``STACK_TYPE'' must be one of: {stack_types}.
    The default value is `IPV4_ONLY`.
  """

  if support_igmp_query:
    multiple_network_interface_cards_spec['igmp-query'] = (
        instances_flags.ValidateNetworkInterfaceIgmpQuery
    )
    network_interface_help += """
      *igmp-query*::: Determines if the Compute Engine Instance can receive and respond to IGMP query packets on the specified network interface.
      ``IGMP_QUERY'' must be one of: `IGMP_QUERY_V2`, `IGMP_QUERY_DISABLED`.
      It is disabled by default.
    """

  parser.add_argument(
      '--network-interface',
      type=arg_parsers.ArgDict(
          spec=multiple_network_interface_cards_spec,
          allow_key_only=True,
      ),
      action='append',  # pylint:disable=protected-access
      metavar='PROPERTY=VALUE',
      help=network_interface_help,
  )


def ValidateInstanceFlexibilityArgs(args):
  """Validates args supplied to --instance-selection-machine-types."""
  flexibility_policy_args = [
      'instance_selection_machine_types',
      'instance_selection',
  ]
  specified_args = []
  for arg in flexibility_policy_args:
    if args.IsKnownAndSpecified(arg):
      specified_args.append(arg)

  if not specified_args:
    return

  if not args.IsSpecified('region'):
    raise exceptions.RequiredArgumentException(
        '--region',
        'Flag --region is required for any of the instance flexibility policy'
        ' flags.',
    )
  if args.IsSpecified('instance_selection'):
    for instance_selection in args.instance_selection:
      if 'name' not in instance_selection:
        raise exceptions.InvalidArgumentException(
            'instance_selection', 'Missing instance selection name.'
        )
      if (
          'machine-type' not in instance_selection
          or not instance_selection['machine-type']
      ):
        raise exceptions.InvalidArgumentException(
            'instance_selection', 'Missing machine type in instance selection.'
        )


def AddCommonBulkInsertArgs(
    parser,
    release_track,
    support_display_device=False,
    support_numa_node_count=False,
    add_zone_region_flags=True,
    support_snp_svsm=False,
    support_max_count_per_zone=False,
    support_custom_hostnames=False,
    support_specific_then_x_affinity=False,
    support_any_reservation_then_fail_affinity=False,
    support_watchdog_timer=False,
    support_igmp_query=False,
    support_graceful_shutdown=False,
    support_flex_start=False,
    support_source_snapshot_region=False,
    support_skip_guest_os_shutdown=False,
    support_preemption_notice_duration=False,
    support_instance_flexibility_policy=False,
    support_workload_identity_config=False,
    support_instance_selection_min_cpu_platform=False,
    support_vsock_mode=False,
):
  """Register parser args common to all tracks."""
  metadata_utils.AddMetadataArgs(parser)
  AddDiskArgsForBulk(parser)
  instances_flags.AddCreateDiskArgs(
      parser,
      enable_kms=True,
      enable_snapshots=True,
      image_csek=True,
      include_name=False,
      support_boot=True,
      support_source_snapshot_region=support_source_snapshot_region,
  )
  instances_flags.AddCanIpForwardArgs(parser)
  instances_flags.AddAcceleratorArgs(parser)
  instances_flags.AddMachineTypeArgs(parser)
  instances_flags.AddMaintenancePolicyArgs(parser, deprecate=True)
  instances_flags.AddNoRestartOnFailureArgs(parser)
  if support_vsock_mode:
    instances_flags.AddVsockModeArgs(parser)
  instances_flags.AddPreemptibleVmArgs(parser)
  instances_flags.AddProvisioningModelVmArgs(
      parser,
      support_flex_start=support_flex_start,
  )
  if support_graceful_shutdown:
    instances_flags.AddGracefulShutdownArgs(parser, is_create=True)

  if support_preemption_notice_duration:
    instances_flags.AddPreemptionNoticeDurationArgs(parser)
  instances_flags.AddNetworkPerformanceConfigsArgs(parser)
  instances_flags.AddInstanceTerminationActionVmArgs(parser)
  instances_flags.AddServiceAccountAndScopeArgs(
      parser,
      False,
      extra_scopes_help=(
          'However, if neither `--scopes` nor `--no-scopes` are '
          'specified and the project has no default service '
          'account, then the instance will be created with no '
          'scopes. Note that the level of access that a service '
          'account has is determined by a combination of access '
          'scopes and IAM roles so you must configure both '
          'access scopes and IAM roles for the service account '
          'to work properly.'
      ),
  )
  instances_flags.AddTagsArgs(parser)
  instances_flags.AddCustomMachineTypeArgs(parser)
  instances_flags.AddNoAddressArg(parser)
  instances_flags.AddNetworkArgs(parser)
  instances_flags.AddNetworkTierArgs(parser, instance=True)
  AddBulkCreateNetworkingArgs(
      parser,
      support_igmp_query=support_igmp_query,
  )

  instances_flags.AddImageArgs(
      parser,
      enable_snapshots=True,
      support_source_snapshot_region=support_source_snapshot_region,
  )
  instances_flags.AddShieldedInstanceConfigArgs(parser)
  instances_flags.AddNestedVirtualizationArgs(parser)
  instances_flags.AddThreadsPerCoreArgs(parser)
  instances_flags.AddEnableUefiNetworkingArgs(parser)
  instances_flags.AddResourceManagerTagsArgs(parser)
  if support_numa_node_count:
    instances_flags.AddNumaNodeCountArgs(parser)

  if support_display_device:
    instances_flags.AddDisplayDeviceArg(parser)

  instances_flags.AddReservationAffinityGroup(
      parser,
      group_text='Specifies the reservation for the instance.',
      affinity_text='The type of reservation for the instance.',
      support_specific_then_x_affinity=support_specific_then_x_affinity,
      support_any_reservation_then_fail_affinity=support_any_reservation_then_fail_affinity,
  )

  maintenance_flags.AddResourcePoliciesArgs(parser, 'added to', 'instance')

  instances_flags.AddMinNodeCpuArg(parser)

  instances_flags.AddLocationHintArg(parser)

  compute_flags.AddEraseVssSignature(
      parser, 'source snapshots or source machine image'
  )

  labels_util.AddCreateLabelsFlags(parser)

  parser.add_argument(
      '--description', help='Specifies a textual description of the instances.'
  )

  base.ASYNC_FLAG.AddToParser(parser)
  parser.display_info.AddFormat(
      'multi(instances:format="table(name,zone.basename())")'
  )

  instances_flags.AddVisibleCoreCountArgs(parser)

  instances_flags.AddLocalSsdArgsWithSize(parser)

  instances_flags.AddMaxRunDurationVmArgs(parser)
  instances_flags.AddDiscardLocalSsdVmArgs(parser)

  AddDistributionTargetShapeArgs(parser)

  instances_flags.AddStackTypeArgs(parser, support_ipv6_only=True)
  instances_flags.AddMinCpuPlatformArgs(parser, release_track)
  instances_flags.AddPublicDnsArgs(parser, instance=True)
  instances_flags.AddConfidentialComputeArgs(
      parser,
      support_confidential_compute_type=True,
      support_confidential_compute_type_tdx=True,
      support_snp_svsm=support_snp_svsm,
  )
  instances_flags.AddPostKeyRevocationActionTypeArgs(parser)
  AddBulkCreateArgs(
      parser,
      add_zone_region_flags,
      support_max_count_per_zone,
      support_custom_hostnames,
  )

  instances_flags.AddPerformanceMonitoringUnitArgs(parser)
  if support_watchdog_timer:
    instances_flags.AddWatchdogTimerArg(parser)
  instances_flags.AddTurboModeArgs(parser)
  if support_skip_guest_os_shutdown:
    instances_flags.AddSkipGuestOsShutdownArgs(parser)
  if support_instance_flexibility_policy:
    compute_flags.AddInstanceFlexibilityPolicyArgs(
        parser,
        support_instance_selection_min_cpu_platform=support_instance_selection_min_cpu_platform,
    )
  if support_workload_identity_config:
    instances_flags.AddWorkloadIdentityConfigArgs(parser)


def ValidateBulkCreateArgs(args):
  """Validates args for bulk create."""
  if args.IsSpecified('name_pattern') and not args.IsSpecified('count'):
    raise exceptions.RequiredArgumentException(
        '--count',
        """The `--count` argument must be specified when the `--name-pattern` argument is specified.""",
    )
  if args.IsSpecified('location_policy') and (
      args.IsSpecified('zone') or not args.IsSpecified('region')
  ):
    raise exceptions.RequiredArgumentException(
        '--region',
        """The `--region` argument must be used alongside the `--location-policy` argument and not `--zone`.""",
    )


def ValidateBulkTargetShapeArgs(args):
  """Validates target shape arg for bulk create."""
  if args.IsSpecified('target_distribution_shape') and (
      args.IsSpecified('zone') or not args.IsSpecified('region')
  ):
    raise exceptions.RequiredArgumentException(
        '--region',
        """The `--region` argument must be used alongside the `--target_distribution_shape` argument and not `--zone`.""",
    )


def ValidateLocationPolicyArgs(args):
  """Validates args supplied to --location-policy."""
  if args.IsSpecified('location_policy'):
    for zone, policy in args.location_policy.items():
      zone_split = zone.split('-')
      if (
          len(zone_split) != 3
          or not zone_split[1][-1].isdigit()
      ):
        raise exceptions.InvalidArgumentException(
            '--location-policy', 'Key [{}] must be a zone.'.format(zone)
        )

      if policy not in ['allow', 'deny']:
        raise exceptions.InvalidArgumentException(
            '--location-policy',
            'Value [{}] must be one of [allow, deny]'.format(policy),
        )


def ValidateMaxCountPerZoneArgs(args):
  """Validates args supplied to --max-count-per-zone."""
  if args.IsKnownAndSpecified('max_count_per_zone'):
    for zone, count in args.max_count_per_zone.items():
      if not ValidateZone(zone):
        raise exceptions.InvalidArgumentException(
            '--max-count-per-zone', 'Key [{}] must be a zone.'.format(zone)
        )
      if not ValidateNaturalCount(count):
        raise exceptions.InvalidArgumentException(
            '--max-count-per-zone',
            'Value [{}] must be a positive natural number.'.format(count),
        )


def ValidateCustomHostnames(args):
  """Validates args supplied to --per-instance-hostnames."""
  if args.IsKnownAndSpecified('per_instance_hostnames'):
    if not args.IsKnownAndSpecified('predefined_names'):
      raise exceptions.RequiredArgumentException(
          '--per-instance-hostnames',
          """The `--per-instance-hostnames` argument must be used alongside the `--predefined-names` argument.""",
      )
    for instance_name, _ in args.per_instance_hostnames.items():
      if instance_name not in args.predefined_names:
        raise exceptions.InvalidArgumentException(
            '--per-instance-hostnames',
            'Instance [{}] missing in predefined_names. Instance names from'
            ' --per-instance-hostnames must be included in --predefined-names'
            ' flag.'.format(instance_name),
        )


def ValidateZone(zone):
  """Validates if zone is valid."""
  return (
      len(zone) < 64 and re.compile(r'^\w+-\w+\d+-\w+').match(zone) is not None
  )


def ValidateNaturalCount(count):
  """Validates if count is positive natural number."""
  return re.compile(r'^[1-9]\d*').match(count) is not None


def ValidateBulkInsertArgs(
    args,
    support_max_count_per_zone,
    support_custom_hostnames,
    support_instance_flexibility_policy,
):
  """Validates all bulk and instance args."""
  ValidateBulkCreateArgs(args)
  ValidateBulkTargetShapeArgs(args)
  ValidateLocationPolicyArgs(args)
  if support_max_count_per_zone:
    ValidateMaxCountPerZoneArgs(args)
  if support_custom_hostnames:
    ValidateCustomHostnames(args)
  ValidateBulkDiskFlags(args)
  instances_flags.ValidateImageFlags(args)
  instances_flags.ValidateLocalSsdFlags(args)
  instances_flags.ValidateNicFlags(args)
  instances_flags.ValidateServiceAccountAndScopeArgs(args)
  instances_flags.ValidateAcceleratorArgs(args)
  instances_flags.ValidateNetworkTierArgs(args)
  instances_flags.ValidateReservationAffinityGroup(args)
  instances_flags.ValidateNetworkPerformanceConfigsArgs(args)
  instances_flags.ValidateInstanceScheduling(
      args, support_max_run_duration=True
  )
  if support_instance_flexibility_policy:
    ValidateInstanceFlexibilityArgs(args)



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Cloud Pub/Sub subscription pull command."""


from apitools.base.py import exceptions as api_ex

from googlecloudsdk.api_lib.pubsub import subscriptions
from googlecloudsdk.api_lib.util import exceptions as util_ex
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.pubsub import flags
from googlecloudsdk.command_lib.pubsub import resource_args
from googlecloudsdk.command_lib.pubsub import util

MESSAGE_FORMAT = """\
table[box](
  message.data.decode(base64).decode(utf-8),
  message.messageId,
  message.orderingKey,
  message.attributes.list(separator='\n'),
  deliveryAttempt,
  ackId.if(NOT auto_ack)
)
"""

MESSAGE_FORMAT_WITH_ACK_STATUS = """\
table[box](
  message.data.decode(base64).decode(utf-8),
  message.messageId,
  message.orderingKey,
  message.attributes.list(separator='\n'),
  deliveryAttempt,
  ackId.if(NOT auto_ack),
  ackStatus.if(auto_ack)
)
"""


def _Run(args, max_messages, return_immediately=False):
  """Pulls messages from a subscription."""
  client = subscriptions.SubscriptionsClient()

  subscription_ref = args.CONCEPTS.subscription.Parse()
  pull_response = client.Pull(subscription_ref, max_messages,
                              return_immediately)

  failed_ack_ids = {}
  ack_ids_and_failure_reasons = []
  if args.auto_ack and pull_response.receivedMessages:
    ack_ids = [message.ackId for message in pull_response.receivedMessages]
    try:
      client.Ack(ack_ids, subscription_ref)
    except api_ex.HttpError as error:
      exc = util_ex.HttpException(error)
      ack_ids_and_failure_reasons = util.ParseExactlyOnceErrorInfo(
          exc.payload.details)
      # If the failure doesn't have more information (specifically for exactly
      # once related failures), assume all the ack ids have failed with the
      # same status.
      if not ack_ids_and_failure_reasons:
        for ack_id in ack_ids:
          failed_ack_ids[ack_id] = 'FAILURE_' + str(error.status_code)

    if not failed_ack_ids:
      for ack_ids_and_failure_reason in ack_ids_and_failure_reasons:
        failed_ack_ids[ack_ids_and_failure_reason[
            'AckId']] = ack_ids_and_failure_reason['FailureReason']

  if not args.auto_ack:
    return pull_response.receivedMessages

  # We attempted to auto-ack this message. Augment the response with ackStatus.
  return_val = []
  for message in pull_response.receivedMessages:
    # Copy the message into a separate object so we can mutate it.
    message_copy = {}
    for field in message.all_fields():
      value = getattr(message, field.name)
      if value:
        message_copy[field.name] = value
    if message.ackId in failed_ack_ids:
      message_copy['ackStatus'] = failed_ack_ids[message.ackId]
    else:
      message_copy['ackStatus'] = 'SUCCESS'
    return_val.append(message_copy)
  return return_val


@base.ReleaseTracks(base.ReleaseTrack.GA)
class Pull(base.ListCommand):
  """Pulls one or more Cloud Pub/Sub messages from a subscription."""

  detailed_help = {
      'DESCRIPTION':
          """\
          Returns one or more messages from the specified Cloud Pub/Sub
          subscription, if there are any messages enqueued.

          By default, this command returns only one message from the
          subscription. Use the `--limit` flag to specify the max messages to
          return.

          Please note that this command is not guaranteed to return all the
          messages in your backlog or the maximum specified in the --limit
          argument.  Receiving fewer messages than available occasionally
          is normal."""
  }

  @staticmethod
  def Args(parser):
    parser.display_info.AddFormat(MESSAGE_FORMAT_WITH_ACK_STATUS)
    resource_args.AddSubscriptionResourceArg(parser, 'to pull messages from.')
    flags.AddPullFlags(parser)

    base.LIMIT_FLAG.SetDefault(parser, 1)

  def Run(self, args):
    return _Run(args, args.limit)


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA)
class PullBeta(Pull):
  """Pulls one or more Cloud Pub/Sub messages from a subscription."""

  @staticmethod
  def Args(parser):
    parser.display_info.AddFormat(MESSAGE_FORMAT_WITH_ACK_STATUS)
    resource_args.AddSubscriptionResourceArg(parser, 'to pull messages from.')
    flags.AddPullFlags(
        parser, add_deprecated=True, add_wait=True, add_return_immediately=True
    )

  def Run(self, args):
    if args.IsSpecified('limit'):
      if args.IsSpecified('max_messages'):
        raise exceptions.ConflictingArgumentsException('--max-messages',
                                                       '--limit')
      max_messages = args.limit
    else:
      max_messages = args.max_messages

    return_immediately = False
    if args.IsSpecified('return_immediately'):
      return_immediately = args.return_immediately
    elif args.IsSpecified('wait'):
      return_immediately = not args.wait

    return _Run(args, max_messages, return_immediately)



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""managed-instance-groups list-instances command.

It's an alias for the instance-groups list-instances command.
"""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import instance_groups_utils
from googlecloudsdk.api_lib.compute import request_helper
from googlecloudsdk.api_lib.compute import utils
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import flags
from googlecloudsdk.command_lib.compute import scope as compute_scope
from googlecloudsdk.command_lib.compute.instance_groups import flags as instance_groups_flags


@base.UniverseCompatible
@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.PREVIEW)
class ListInstances(base.ListCommand):
  """List Compute Engine instances present in managed instance group."""

  @staticmethod
  def Args(parser):
    instance_groups_flags.AddListInstancesOutputFormat(parser)
    parser.display_info.AddUriFunc(
        instance_groups_utils.UriFuncForListInstanceRelatedObjects)
    instance_groups_flags.MULTISCOPE_INSTANCE_GROUP_MANAGER_ARG.AddArgument(
        parser)

  def Run(self, args):
    managed_instances = self.GetManagedInstances(args)
    # if uri and format are not provided by the user, then change the output to
    # hide dynamic fields if they are not present in fetched instances
    if not args.uri and not args.IsSpecified('format'):
      args.format = (
          instance_groups_flags.GetListInstancesOutputWithDynamicFields(
              managed_instances, self.ReleaseTrack()
          )
      )
    return managed_instances

  def GetManagedInstances(self, args):
    """Retrieves response with instance in the instance group."""
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client

    group_ref = (instance_groups_flags.MULTISCOPE_INSTANCE_GROUP_MANAGER_ARG.
                 ResolveAsResource(
                     args,
                     holder.resources,
                     default_scope=compute_scope.ScopeEnum.ZONE,
                     scope_lister=flags.GetDefaultScopeLister(client)))

    if hasattr(group_ref, 'zone'):
      service = client.apitools_client.instanceGroupManagers
      request = (client.messages.
                 ComputeInstanceGroupManagersListManagedInstancesRequest(
                     instanceGroupManager=group_ref.Name(),
                     zone=group_ref.zone,
                     project=group_ref.project))
    elif hasattr(group_ref, 'region'):
      service = client.apitools_client.regionInstanceGroupManagers
      request = (client.messages.
                 ComputeRegionInstanceGroupManagersListManagedInstancesRequest(
                     instanceGroupManager=group_ref.Name(),
                     region=group_ref.region,
                     project=group_ref.project))

    errors = []
    results = list(request_helper.MakeRequests(
        requests=[(service, 'ListManagedInstances', request)],
        http=client.apitools_client.http,
        batch_url=client.batch_url,
        errors=errors))

    if errors:
      utils.RaiseToolException(errors)

    return results


ListInstances.detailed_help = {
    'brief':
        'List instances present in the managed instance group',
    'DESCRIPTION':
        """\
          *{command}* lists instances in a managed instance group.

          The required permission to execute this command is
          `compute.instanceGroupManagers.list`. If needed, you can include this
          permission, or choose any of the following preexisting IAM roles
          that contain this particular permission:

          *   Compute Admin
          *   Compute Viewer
          *   Compute Instance Admin (v1)
          *   Compute Instance Admin (beta)
          *   Compute Network Admin
          *   Editor
          *   Owner
          *   Security Reviewer
          *   Viewer

          For more information regarding permissions required by managed
          instance groups, refer to Compute Engine's access control guide :
          https://cloud.google.com/compute/docs/access/iam#managed-instance-groups-and-iam.
        """,
    'EXAMPLES':
        """\
        To see additional details about the instances in a managed instance
        group `my-group`, including per-instance overrides, run:

            $ {command} \\
                  my-group --format=yaml
        """
}


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class ListInstancesBeta(ListInstances):
  """List Compute Engine instances present in managed instance group."""

  @staticmethod
  def Args(parser):
    instance_groups_flags.AddListInstancesOutputFormat(
        parser, release_track=base.ReleaseTrack.BETA)
    parser.display_info.AddUriFunc(
        instance_groups_utils.UriFuncForListInstanceRelatedObjects)
    instance_groups_flags.MULTISCOPE_INSTANCE_GROUP_MANAGER_ARG.AddArgument(
        parser)


ListInstancesBeta.detailed_help = ListInstances.detailed_help


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class ListInstancesAlpha(ListInstancesBeta):
  """List Compute Engine instances present in managed instance group."""

  @staticmethod
  def Args(parser):
    instance_groups_flags.AddListInstancesOutputFormat(
        parser, release_track=base.ReleaseTrack.ALPHA)
    parser.display_info.AddUriFunc(
        instance_groups_utils.UriFuncForListInstanceRelatedObjects)
    instance_groups_flags.MULTISCOPE_INSTANCE_GROUP_MANAGER_ARG.AddArgument(
        parser)


ListInstancesAlpha.detailed_help = ListInstancesBeta.detailed_help



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Updates Backup and DR Backup Plan Association."""


from apitools.base.py import exceptions as apitools_exceptions
from googlecloudsdk.api_lib.backupdr import util
from googlecloudsdk.api_lib.backupdr.backup_plan_associations import BackupPlanAssociationsClient
from googlecloudsdk.api_lib.util import exceptions
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.backupdr import flags
from googlecloudsdk.core import log


@base.UniverseCompatible
@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA, base.ReleaseTrack.GA
)
class Update(base.UpdateCommand):
  """Update a specific backup plan within a backup plan association."""

  detailed_help = {
      'BRIEF': (
          'Update a specific backup plan within a backup plan association.'
      ),
      'DESCRIPTION': (
          '{description}'
      ),
      'EXAMPLES': """\
        To update backup plan association `sample-bpa` in project `sample-project` and location `us-central1` with backup plan `sample-backup-plan` in the same project and location, run:

          $ {command} sample-bpa --project=sample-project --location=us-central1 --backup-plan=sample-backup-plan

        To update backup plan association `sample-bpa-uri` with backup plan `sample-backup-plan-uri` (using full URIs), run:

          $ {command} sample-bpa-uri --backup-plan=sample-backup-plan-uri

        To update backup plan association `sample-bpa` in location `us-central1` with backup plan `sample-backup-plan-uri`, run:

          $ {command} sample-bpa --location=us-central1 --backup-plan=sample-backup-plan-uri

        To update backup plan association `sample-bpa` in project `workload-project` and location `us-central1` with backup plan `sample-backup-plan` in project `sample-project`, run:

          $ {command} sample-bpa --workload-project=workload-project --location=us-central1 --backup-plan=sample-backup-plan --project=sample-project
        """,
  }

  @staticmethod
  def Args(parser):
    """Specifies additional command flags.

    Args:
      parser: argparse.Parser: Parser object for command line inputs.
    """
    base.ASYNC_FLAG.AddToParser(parser)
    base.ASYNC_FLAG.SetDefault(parser, True)
    flags.AddUpdateBackupPlanAssociationFlags(parser)

  def GetUpdateMask(self, args):
    return 'backup_plan' if args.IsSpecified('backup_plan') else ''

  def Run(self, args):
    """Constructs and sends request.

    Args:
      args: argparse.Namespace, An object that contains the values for the
        arguments specified in the .Args() method.

    Returns:
      ProcessHttpResponse of the request made.
    """
    api_version = util.GetApiVersion(self.ReleaseTrack())
    client = BackupPlanAssociationsClient(api_version=api_version)
    is_async = args.async_

    backup_plan_association = args.CONCEPTS.backup_plan_association.Parse()
    backup_plan = args.CONCEPTS.backup_plan.Parse()

    try:
      parsed_bpa = client.ParseUpdate(backup_plan)
      update_mask = self.GetUpdateMask(args)
      operation = client.Update(
          backup_plan_association,
          parsed_bpa,
          update_mask=update_mask,
      )
    except apitools_exceptions.HttpError as e:
      raise exceptions.HttpException(e, util.HTTP_ERROR_FORMAT) from e

    if is_async:
      log.UpdatedResource(
          backup_plan_association.RelativeName(),
          kind='backup plan association',
          is_async=True,
          details=util.ASYNC_OPERATION_MESSAGE.format(operation.name),
      )
      return operation

    resource = client.WaitForOperation(
        operation_ref=client.GetOperationRef(operation),
        message=(
            'Updating backup plan association [{}].  (This operation could'
            ' take up to 2 minutes.)'.format(
                backup_plan_association.RelativeName()
            )
        ),
    )
    log.UpdatedResource(
        backup_plan_association.RelativeName(), kind='backup plan association'
    )
    return resource



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Updates a Backup and DR Backup Vault."""

import argparse

from apitools.base.py import exceptions as apitools_exceptions
from googlecloudsdk.api_lib.backupdr import util
from googlecloudsdk.api_lib.backupdr.backup_vaults import BackupVaultsClient
from googlecloudsdk.api_lib.util import exceptions
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions as calliope_exceptions
from googlecloudsdk.command_lib.backupdr import flags
from googlecloudsdk.command_lib.backupdr import util as command_util
from googlecloudsdk.core import log


def _add_common_args(parser: argparse.ArgumentParser):
  """Specifies additional command flags.

  Args:
    parser: argparse.Parser: Parser object for command line inputs.
  """
  flags.AddBackupVaultResourceArg(
      parser,
      'Name of the existing backup vault to update.',
  )
  flags.AddNoAsyncFlag(parser)
  flags.AddEnforcedRetention(parser, False)
  flags.AddDescription(parser)
  flags.AddEffectiveTime(parser)
  flags.AddUnlockBackupMinEnforcedRetention(parser)
  flags.AddForceUpdateFlag(parser)
  flags.AddForceUpdateAccessRestriction(parser)
  flags.AddBackupVaultAccessRestrictionEnumFlag(parser, 'update')


def _add_common_update_mask(args: argparse.Namespace) -> str:
  """Creates the update mask for the update command.

  Args:
    args: argparse.Namespace, An object that contains the values for the
      arguments specified in the .Args() method.

  Returns:
    A string containing the update mask.
  """
  updated_fields = []
  if args.IsSpecified('description'):
    updated_fields.append('description')
  if args.IsSpecified('backup_min_enforced_retention'):
    updated_fields.append('backupMinimumEnforcedRetentionDuration')
  if args.IsSpecified(
      'unlock_backup_min_enforced_retention'
  ) or args.IsSpecified('effective_time'):
    updated_fields.append('effectiveTime')
  if args.IsSpecified('access_restriction'):
    updated_fields.append('accessRestriction')
  return ','.join(updated_fields)


def _parse_update_bv(args: argparse.Namespace):
  """Parses the update backup vault arguments.

  Args:
    args: argparse.Namespace, An object that contains the values for the
      arguments specified in the .Args() method.

  Returns:
    A tuple containing the backup min enforced retention, description and
    effective time.

  Raises:
    calliope_exceptions.ConflictingArgumentsException: If both
      --unlock-backup-min-enforced-retention and --effective-time are specified.
  """
  backup_min_enforced_retention = command_util.ConvertIntToStr(
      args.backup_min_enforced_retention
  )
  description = args.description

  if args.unlock_backup_min_enforced_retention and args.effective_time:
    raise calliope_exceptions.ConflictingArgumentsException(
        'Only one of --unlock-backup-min-enforced-retention or '
        '--effective-time can be specified.'
    )

  if args.unlock_backup_min_enforced_retention:
    effective_time = command_util.ResetEnforcedRetention()
  else:
    effective_time = command_util.VerifyDateInFuture(
        args.effective_time, 'effective-time'
    )
  if args.access_restriction:
    access_restriction = args.access_restriction
  else:
    access_restriction = None

  return (
      backup_min_enforced_retention,
      description,
      effective_time,
      access_restriction,
  )


def _run(
    self,
    args: argparse.Namespace,
):
  """Constructs and sends request.

  Args:
    self: The object that is calling this method.
    args: argparse.Namespace, An object that contains the values for the
      arguments specified in the .Args() method.

  Returns:
    ProcessHttpResponse of the request made.

  Raises:
    exceptions.HttpException: if the http request fails.
  """
  api_version = util.GetApiVersion(self.ReleaseTrack())
  client = BackupVaultsClient(api_version=api_version)
  backup_vault = args.CONCEPTS.backup_vault.Parse()
  no_async = args.no_async
  force_update_access_restriction = args.force_update_access_restriction

  try:
    parsed_bv = self.ParseUpdateBv(args, client)

    update_mask = self.GetUpdateMask(args)

    operation = client.Update(
        backup_vault,
        parsed_bv,
        force_update=args.force_update,
        update_mask=update_mask,
        force_update_access_restriction=force_update_access_restriction,
    )

  except apitools_exceptions.HttpError as e:
    raise exceptions.HttpException(e, util.HTTP_ERROR_FORMAT)

  if no_async:
    resource = client.WaitForOperation(
        operation_ref=client.GetOperationRef(operation),
        message=(
            'Updating backup vault [{}]. (This operation could'
            ' take up to 1 minute.)'.format(backup_vault.RelativeName())
        ),
        has_result=False,
    )
    log.UpdatedResource(backup_vault.RelativeName(), kind='backup vault')
    return resource

  log.UpdatedResource(
      backup_vault.RelativeName(),
      kind='backup vault',
      is_async=True,
      details=util.ASYNC_OPERATION_MESSAGE.format(operation.name),
  )
  return operation


@base.ReleaseTracks(base.ReleaseTrack.GA)
@base.DefaultUniverseOnly
class Update(base.UpdateCommand):
  """Update a Backup and DR backup vault."""

  detailed_help = {
      'BRIEF': 'Updates a Backup and DR backup vault.',
      'DESCRIPTION': '{description}',
      'API REFERENCE': (
          'This command uses the backupdr/v1 API. The full documentation for'
          ' this API can be found at:'
          ' https://cloud.google.com/backup-disaster-recovery'
      ),
      'EXAMPLES': """\
        To update a backup vault BACKUP_VAULT in location MY_LOCATION with one update
        field, run:

        $ {command} BACKUP_VAULT --location=MY_LOCATION --effective-time="2024-03-22"

        To update a backup vault BACKUP_VAULT in location MY_LOCATION with multiple
        update fields, run:

        $ {command} BACKUP_VAULT --location=MY_LOCATION \
            --backup-min-enforced-retention="400000s" --description="Updated backup vault"
        """,
  }

  @staticmethod
  def Args(parser: argparse.ArgumentParser):
    """Specifies additional command flags.

    Args:
      parser: argparse.Parser: Parser object for command line inputs.
    """
    _add_common_args(parser)

  def GetUpdateMask(self, args: argparse.Namespace) -> str:
    """Returns the update mask for the update command.

    Args:
      args: argparse.Namespace, An object that contains the values for the
        arguments specified in the .Args() method.

    Returns:
      A string containing the update mask.
    """
    return _add_common_update_mask(args)

  def ParseUpdateBv(self, args: argparse.Namespace, client: BackupVaultsClient):
    """Parses the update backup vault arguments.

    Args:
      args: argparse.Namespace, An object that contains the values for the
        arguments specified in the .Args() method.
      client: BackupVaultsClient, The client to use to parse the backup vault.

    Returns:
      A parsed backup vault object.
    """
    (
        backup_min_enforced_retention,
        description,
        effective_time,
        access_restriction,
    ) = _parse_update_bv(args)

    return client.ParseUpdate(
        description=description,
        backup_min_enforced_retention=backup_min_enforced_retention,
        effective_time=effective_time,
        access_restriction=access_restriction,
    )

  def Run(self, args):
    return _run(self, args)


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class UpdateAlpha(Update):
  """Update a Backup and DR backup vault."""

  @staticmethod
  def Args(parser):
    """Specifies additional command flags.

    Args:
      parser: argparse.Parser: Parser object for command line inputs.
    """
    _add_common_args(parser)

  def ParseUpdateBv(self, args: argparse.Namespace, client: BackupVaultsClient):
    """Parses the update backup vault arguments.

    Args:
      args: argparse.Namespace, An object that contains the values for the
        arguments specified in the .Args() method.
      client: BackupVaultsClient, The client to use to parse the backup vault.

    Returns:
      A parsed backup vault object.
    """
    (
        backup_min_enforced_retention,
        description,
        effective_time,
        access_restriction,
    ) = _parse_update_bv(args)

    return client.ParseUpdate(
        description=description,
        backup_min_enforced_retention=backup_min_enforced_retention,
        effective_time=effective_time,
        access_restriction=access_restriction,
    )

  def GetUpdateMask(self, args: argparse.Namespace) -> str:
    """Returns the update mask for the update command.

    Args:
      args: argparse.Namespace, An object that contains the values for the
        arguments specified in the .Args() method.

    Returns:
      A string containing the update mask.
    """
    mask = _add_common_update_mask(args)
    return mask

  def Run(self, args):
    return _run(self, args)


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class UpdateBeta(Update):
  """Update a Backup and DR backup vault."""

  @staticmethod
  def Args(parser):
    """Specifies additional command flags.

    Args:
      parser: argparse.Parser: Parser object for command line inputs.
    """
    _add_common_args(parser)

  def ParseUpdateBv(self, args: argparse.Namespace, client: BackupVaultsClient):
    """Parses the update backup vault arguments.

    Args:
      args: argparse.Namespace, An object that contains the values for the
        arguments specified in the .Args() method.
      client: BackupVaultsClient, The client to use to parse the backup vault.

    Returns:
      A parsed backup vault object.
    """
    (
        backup_min_enforced_retention,
        description,
        effective_time,
        access_restriction,
    ) = _parse_update_bv(args)

    return client.ParseUpdate(
        description=description,
        backup_min_enforced_retention=backup_min_enforced_retention,
        effective_time=effective_time,
        access_restriction=access_restriction,
    )

  def GetUpdateMask(self, args: argparse.Namespace) -> str:
    """Returns the update mask for the update command.

    Args:
      args: argparse.Namespace, An object that contains the values for the
        arguments specified in the .Args() method.

    Returns:
      A string containing the update mask.
    """
    mask = _add_common_update_mask(args)
    return mask

  def Run(self, args):
    return _run(self, args)
